package ms.itsecteam.warpdrink.manager;

import android.app.AlertDialog;
import android.content.DialogInterface;
import android.util.Log;
import android.widget.Toast;

import java.util.Arrays;
import java.util.List;
import java.util.ListIterator;

import ms.itsecteam.warpdrink.MainActivity;
import ms.itsecteam.warpdrink.R;
import ms.itsecteam.warpdrink.data.objects.User;
import ms.itsecteam.warpdrink.data.rest.RestClient;
import ms.itsecteam.warpdrink.orderList.Order;

public class MainManager {

    private MainActivity mainActivity;
    private DatabaseManager databaseManager;
    private UiManager uiManager;
    private DataManager dataManager;

    private static MainManager ourInstance = new MainManager();
    public static MainManager getInstance() {
        return ourInstance;
    }

    private MainManager() {
        this.databaseManager = DatabaseManager.getInstance();
        this.uiManager = UiManager.getInstance();
        this.dataManager = DataManager.getInstance();
    }

    public void init(MainActivity mainActivity) {
        this.mainActivity = mainActivity;
        this.uiManager.initUi(this.mainActivity);
        this.refreshUserData();
    }

    public void refreshUserData() {
        List<User> userList = RestClient.getInstance().getAllUser();
        this.databaseManager.refreshUserData(userList);
        if(userList != null) {
            this.uiManager.refreshUserData(userList);
        }
    }

    public boolean addUser(String name) {
         if(this.databaseManager.getUserByName(name) == null) {
                User u = new User();
                u.setName(name);
                RestClient.getInstance().createUser(u);
                this.refreshUserData();
                return true;
        }
        return false;
    }

    public void setCurrentUser(User user) {
        if (user != null) {
            this.dataManager.setCurrentUser(user);
            this.uiManager.setCurrentUser(user);
        }
    }

    public void clearCurrentUser() {
        this.dataManager.clearCurrentUser();
        this.uiManager.clearCurrentUser();
    }
    public void addOrder(double amount) {
        DataManager.getInstance().addToTotalAmount(amount);
        this.uiManager.refreshTotalTextView(DataManager.getInstance().getTotalAmount());
    }

    public void removeOrder(Order order) {
        DataManager.getInstance().addToTotalAmount((-1)*order.getValue());
        this.uiManager.removeOrder(order);
        this.uiManager.refreshTotalTextView(DataManager.getInstance().getTotalAmount());

    }
    public void chargeAmount(double amount) {
        User user = this.dataManager.getCurrentUser();
        user.setCredit(user.getCredit()+amount);
        user.save();
        RestClient.getInstance().saveUserCredit(user);
        this.uiManager.refreshCreditTextView(user.getCredit());
    }
    public void performPayment() {
        User user = DataManager.getInstance().getCurrentUser();
        double totalAmount = DataManager.getInstance().getTotalAmount();

        if (totalAmount > 0.0 && user != null){
            if (DataManager.getInstance().getCurrentUser().getCredit()>=totalAmount){
                new AlertDialog.Builder(mainActivity)
                        .setTitle(mainActivity.getResources().getString(R.string.confirm_payment_title))
                        .setMessage(mainActivity.getResources().getString(R.string.confirm_payment, totalAmount))
                        .setPositiveButton(android.R.string.yes, new DialogInterface.OnClickListener() {
                            public void onClick(DialogInterface dialog, int which) {
                                DataManager.getInstance().performPayment();
                                MainManager.getInstance().clearCurrentUser();

                            }
                        })
                        .setNegativeButton(android.R.string.no, new DialogInterface.OnClickListener() {
                            public void onClick(DialogInterface dialog, int which) {
                            }
                        })
                        .setIcon(android.R.drawable.ic_dialog_alert)
                        .show();
            } else {
                Toast.makeText(mainActivity, mainActivity.getResources().getString(R.string.please_charge), Toast.LENGTH_LONG).show();
            }
        }
    }


    public MainActivity getMainActivity() {
        return mainActivity;
    }

    public void setMainActivity(MainActivity mainActivity) {
        this.mainActivity = mainActivity;
    }


}
