package ms.itsecteam.warpdrink.manager;

import android.util.Log;
import android.view.KeyEvent;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.AutoCompleteTextView;
import android.widget.Button;
import android.widget.ImageButton;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;

import java.util.ArrayList;
import java.util.List;
import java.util.ListIterator;

import ms.itsecteam.warpdrink.MainActivity;
import ms.itsecteam.warpdrink.R;
import ms.itsecteam.warpdrink.data.objects.User;
import ms.itsecteam.warpdrink.dialogs.ChargeDialog;
import ms.itsecteam.warpdrink.dialogs.NewUserDialog;
import ms.itsecteam.warpdrink.dialogs.SettingsDialog;
import ms.itsecteam.warpdrink.orderList.ListViewAdapter;
import ms.itsecteam.warpdrink.orderList.Order;

/**
 * Created by chris on 29.06.2015.
 */
public class UiManager implements View.OnKeyListener, View.OnClickListener, AdapterView.OnItemClickListener {

    private MainActivity mainActivity;
    private Button btnPay, btnCharge, btnLogout;
    private ImageButton btnRefresh;
    private TextView txtSum,txtCredit;
    private ListView lstOrdered;
    private AutoCompleteTextView atxvName;

    private ListViewAdapter lstAdapter;
    private ArrayAdapter atxvAdapter;
    private ArrayList<Double> order;
    private List<User> users;

    private static UiManager ourInstance = new UiManager();

    public static UiManager getInstance() {
        return ourInstance;
    }

    private UiManager() {}

    protected void initUi(MainActivity mainActivity) {
        this.txtSum = (TextView) mainActivity.findViewById(R.id.txtSum);
        this.txtCredit = (TextView)mainActivity.findViewById(R.id.txtCredit);
        this.txtCredit.setText(mainActivity.getResources().getString(R.string.credit,0.0));
        this.txtSum.setText(mainActivity.getResources().getString(R.string.total,0.0));

        this.btnLogout = (Button) mainActivity.findViewById(R.id.btnLogout);
        this.btnRefresh = (ImageButton) mainActivity.findViewById(R.id.btnRefresh);
        this.btnPay = (Button) mainActivity.findViewById(R.id.btnPay);
        this.btnCharge = (Button) mainActivity.findViewById(R.id.btnCharge);

        this.lstAdapter = new ListViewAdapter(mainActivity, R.layout.order_list_item, new ArrayList<Order>());
        this.lstOrdered = (ListView) mainActivity.findViewById(R.id.lstOrdered);
        this.lstOrdered.setAdapter(this.lstAdapter);


        this.users = User.getAll();
        this.atxvAdapter = new ArrayAdapter(mainActivity, android.R.layout.select_dialog_item, this.users);
        this.atxvName = (AutoCompleteTextView) mainActivity.findViewById(R.id.atxvName);
        this.atxvName.setAdapter(atxvAdapter);
        this.atxvName.setThreshold(1);
        this.atxvName.setOnKeyListener(this);
        this.atxvName.setOnItemClickListener(this);

        this.btnLogout.setVisibility(View.INVISIBLE);
        this.enableControls(false);
        this.mainActivity = mainActivity;
    }

    protected void refreshTotalTextView(double totalAmount) {
        this.txtSum.setText(this.mainActivity.getResources().getString(R.string.total,totalAmount));
    }
    protected void refreshCreditTextView(double credit) {
        this.txtCredit.setText(this.mainActivity.getResources().getString(R.string.credit, credit));
    }

    protected void setCurrentUser(User user) {
        this.enableControls(true);
        this.atxvName.dismissDropDown();
        this.btnLogout.setVisibility(View.VISIBLE);
        this.refreshCreditTextView(user.getCredit());
    }

    protected void clearCurrentUser() {
        this.atxvName.setText("");
        this.lstAdapter.clear();
        this.enableControls(false);
        this.btnLogout.setVisibility(View.INVISIBLE);
        this.refreshCreditTextView(0.0);
        this.refreshTotalTextView(0.0);
    }

    private void enableControls(boolean enable) {
        this.atxvName.setEnabled(!enable);
        this.btnRefresh.setEnabled(!enable);
        this.btnCharge.setEnabled(enable);
        this.btnPay.setEnabled(enable);
        this.btnLogout.setEnabled(enable);
    }

    protected void refreshUserData(List<User> userList) {
        this.atxvAdapter.clear();
        ListIterator<User> userListIterator = userList.listIterator();
        while(userListIterator.hasNext()) {
            this.atxvAdapter.add(userListIterator.next());
        }
    }
    protected void removeOrder(Order order) {
        this.lstAdapter.remove(order);
    }

    @Override
    public void onClick(View v) {
        switch (v.getId()) {
            case R.id.btnOneEuro:
                this.lstAdapter.insert(new Order(1.0), this.lstAdapter.getCount());
                MainManager.getInstance().addOrder(1.0);
                break;
            case R.id.btnFiftyCent:
                this.lstAdapter.insert(new Order(0.5), this.lstAdapter.getCount());
                MainManager.getInstance().addOrder(0.5);
                break;
            case R.id.btnCharge:
                new ChargeDialog().show();
                break;
            case R.id.btnPay:
                MainManager.getInstance().performPayment();
                break;
            case R.id.btnLogout:
                MainManager.getInstance().clearCurrentUser();
                break;
            case R.id.btnAddUser:
                new NewUserDialog().show();
                break;
            case R.id.btnRefresh:
                MainManager.getInstance().refreshUserData();
                Toast.makeText(this.mainActivity, "Refresh successful", Toast.LENGTH_LONG).show();
                break;
        }
    }

    @Override
    public boolean onKey(View v, int keyCode, KeyEvent event) {
        if (event.getAction()==KeyEvent.ACTION_DOWN && event.getKeyCode() == KeyEvent.KEYCODE_ENTER) {
            if(this.atxvAdapter.getCount() == 1) {
                MainManager.getInstance().setCurrentUser((User) this.atxvAdapter.getItem(0));
                this.atxvName.setText(this.atxvAdapter.getItem(0).toString());
                this.atxvName.dismissDropDown();
            }
            return true;
        }
        return false;
    }
    @Override
    public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
        MainManager.getInstance().setCurrentUser((User) this.atxvAdapter.getItem(position));
    }
}
